/* If you modify this file, you must re-run Makefile.PL! */

#ifndef hist_constants_h_
#define hist_constants_h_

#define INTEGRAL_CONSTANT    0  /* Sum of rectangular bin areas */
/*#define INTEGRAL_POL1        1*/  /* Use linear functions to interpolate between
                                   bin centers, extrapolate to min/max with the
                                   outermost line */

#endif
